import metrics
import logging

logger = logging.getLogger('fiveruns.dash.recipes')
registry = {}

class Recipe(metrics.MetricSetting):

    def __init__(self, name, url):
        super(Recipe, self).__init__()
        self.name = name
        self.url = url
        self._register()

    def _add_metric(self, *args, **options):
        "Add a metric"
        options['recipe_name'] = self.name
        options['recipe_url'] = self.url
        super(Recipe, self)._add_metric(*args, **options)

    def _register(self):
        "Register this recipe"
        key = (self.name, self.url)
        if key in registry:
            raise DuplicateRecipe(self.name, self.url)
        else:
            registry[key] = self
            logger.debug("Registered recipe `%s' for `%s'" % (self.name, self.url))

    def _replay_recipe(self, recipe):
        logger.debug("Adding %d metric(s) from recipe `%s' for `%s' to recipe `%s' for `%s'" % (
            len(recipe.metrics),
            recipe.name, recipe.url,
            self.name, self.url))
        super(Recipe, self)._replay_recipe(recipe)

def find(name, url):
    "Find a registered recipe"
    if (name, url) in registry:
        return registry[(name, url)]
    elif not url:
        matching = [r for k, r in registry.iteritems() if k[0] == name]
        number = len(matching)
        if number == 1:
            return matching[0]
        elif number > 1:
            raise AmbiguousRecipe(name, [u for (n, u) in matching])    
        else:
            raise UnknownRecipe(name, url)
    else:
        raise UnknownRecipe(name, url)

class RecipeError(NameError):

    def __init__(self, name, urls):
        self.name = name
        self.urls = urls

    def __str__(self):
        return "`%s' defined for `%s'" % (self.name, self.urls)

class UnknownRecipe(NameError):
    """
    Raised when fiveruns.dash.recipes.find cannot find a requested recipe
    """

    def __init__(self, name, url):
        self.name = name
        self.url = url

    def __str__(self):
        return "`%s' for `%s'" % (self.name, self.url)
        
class AmbiguousRecipe(RecipeError):
    """
    Raised when fiveruns.dash.recipes.find matches multiple requested recipes
    """
    pass

class DuplicateRecipe(RecipeError):
    """
    Raised when fiveruns.dash.recipes.find cannot register a recipe due to a duplicate
    """        
    pass



import httplib
import logging
import mimetypes
import os
import sys
import urlparse
import zlib
from socket import gethostname

import simplejson
import scm

logger = logging.getLogger('fiveruns.dash.protocol')

connections = {'http': httplib.HTTPConnection, 'https': httplib.HTTPSConnection}

class Payload(object):

    def __init__(self, config):
        self.config = config
        self.data = self._extract_data()

    def send(self):
        if not self.valid():
            logger.error("Invalid payload format")
            logger.debug(self.data)
            return False
        logger.debug("Sending to %s%s\n%s", self.url(), self.path(), self.data)
        urlparts = urlparse.urlparse(self.url())
        (status, reason, body) = send(
          urlparts,
          self.path(),
          self._extra_params(),
          [('file', 'data.json.gz', self._compressed())]
        )
        if status == 201:
            self._succeeded(body)
            return True
        elif status in range(400, 499):
            print body
            self._failed(reason)
        else:
            print body
            self._unknown(status, reason)
        return False # TODO: HTTP

    def url(self):
        if 'DASH_UPDATE' in os.environ:
            return os.environ['DASH_UPDATE']
        else:
            return 'https://dash-collector.fiveruns.com'

    def valid(self):
        if hasattr(self.__class__, 'validations'):
            for validation in self._class__.validations:
                if not validation(self):
                    return False
        return True

    def _succeeded(self, body):
        logger.debug("Succeeded.")

    def _failed(self, reason):
        logger.debug("Failed (%s)" % reason)

    def _unknown(self, status, reason):
        logger.debug("Unknown error (%s, %s)" % (status, reason))

    def _extra_params(self):
        return [
          ('hostname', gethostname()),
          ('app_id', self.config.app_token),
          ('ip', '127.0.0.1'), # FIXME
          ('pid', str(self.config.pid)),
          ('pwd', self.config.pwd),
          ('vm_version', self.config.vm_version),
          ('process_id', self.config.process_id and str(self.config.process_id) or ''),
        ]

    def _compressed(self):
        return zlib.compress(self._serialize())

    def _serialize(self):
        return simplejson.dumps(self.data)


class InfoPayload(Payload):

    def path(self):
        return "/apps/%s/processes.json" % self.config.app_token

    def _extract_data(self):
        metric_infos = [m.metadata() for m in self.config.metrics.values()]
        recipes = []
        for info in metric_infos:
            key = {"name": info["recipe_name"], "url": info["recipe_url"]}
            if all(v is None for v in key.values()): continue
            if not key in recipes: recipes.append(key)
        data = {"metric_infos" : metric_infos, "recipes" : recipes}
        data.update(self.config.extra_payload_data.get('info', {}))
        return data

    def _extra_params(self):
        params = super(InfoPayload, self)._extra_params()
        return params + self._scm_data()

    def _scm_data(self):
        handler = scm.at(self.config.pwd)
        if handler:
            return dict(handler).items()
        else:
            return []

    def _succeeded(self, body):
        Payload._succeeded(self, body)
        # json = simplejson.loads(body)
        # self.config.process_id = json['process_id']
        # for metric in json['metric_infos']:
        #   for local_metric in self.config.metrics.values():
        #     metadata = local_metric.metadata()
        #     if metric['name'] == metadata['name'] and metric['recipe_name'] == metadata['recipe_name'] and metric['recipe_url'] == metadata['recipe_url']:
        #       logger.debug(metric['id'])
        #       local_metric.info_id = metric['id']
        #       break    

class DataPayload(Payload):

    def path(self):
        return "/apps/%s/metrics.json" % self.config.app_token

    def _extract_data(self):
        data = {}
        real_data = {}
        real_metrics = [(name, metric) for name, metric in self.config.metrics.iteritems() if not metric.virtual]
        virtual_metrics = [(name, metric) for name, metric in  self.config.metrics.iteritems() if metric.virtual]
        for name, metric in real_metrics:
            real_data[name] = self._envelope(metric, metric.values())
        for name, metric in virtual_metrics:
            data[name] = self._envelope(metric, metric.calculate(real_data))
        data.update(real_data)
        return data.values()

    def _envelope(self, m, values):
        """
        Metric data and metadata container
        """
        return {
            "name": m.name,
            "help_text": m.help_text,
            "unit": m.unit,
            "data_type": m.data_type,
            "description": m.description,
            "recipe_name": m.recipe_name,
            "recipe_url": m.recipe_url,
            "name": m.name,
            "values": values }

    
class PingPayload(Payload):
    def path(self):
        return "/apps/%s/ping" % self.config.app_token
  
class TracePayload(Payload):
    def path(self):
        return "/apps/%s/traces.json" % self.config.app_token
  
class ExceptionsPayload(Payload):
    def path(self):
        return "/apps/%s/exceptions" % self.config.app_token
  
def send(urlparts, selector, fields, files):
    content_type, body = encode(fields, files)
    connector = connections[urlparts.scheme]
    connection = connector(urlparts.netloc)
    headers = {'User-Agent': 'Python','Content-Type': content_type}
    try:
        connection.request('POST', selector, body, headers)
    except:
        return [False, sys.exc_info()[1], None]      
    res = connection.getresponse()
    return res.status, res.reason, res.read()
    
def encode(fields, files):
    """
    fields is a sequence of (name, value) elements for regular form fields.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files
    Return (content_type, body) ready for httplib.HTTP instance
    """
    BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
    CRLF = '\r\n'
    L = []
    for (key, value) in fields:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"' % key)
        L.append('')
        L.append(value)
    for (key, filename, value) in files:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
        L.append('Content-Type: %s' % _content_type(filename))
        L.append('')
        L.append(value)
    L.append('--' + BOUNDARY + '--')
    L.append('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
    return content_type, body

def _content_type(filename):
    return mimetypes.guess_type(filename)[0] or 'application/octet-stream'



from __future__ import with_statement
from threading import RLock
import time
import logging
import copy

import aspects

logger = logging.getLogger('fiveruns.dash.metrics')

class MetricError(StandardError):
    pass

class Calculation(object):
    def __init__(self, operation, *fields):
        self.operation = operation
        self.fields = fields

    def run(self, *args):
        return self.operation(*args)

class Metric(object):

    def __init__(self, name, description, **options):
        self.info_id = None
        self.name = name
        self.description = description
        self.options = options
        self.containers = {}
        self.context_finder = None
        self.unit = None
        self.data_type = self._data_type()
        self.recipe_name = options.get('recipe_name', None)
        self.recipe_url = options.get('recipe_url', None)
        self.help_text = None
        self.calculation = options.get('calculation', None)
        self.lock = RLock()
        self._validate()
        logger.debug("Defined %s metric `%s' (recipe `%s' for `%s')" % (self.data_type, self.name, self.recipe_name, self.recipe_url))

    def virtual(self):
        if self.calculation:
            return True
        else:
            return False
    virtual = property(virtual)

    def calculate(self, data):
        """
        Used if this metric is virtual

        Note: Does not support time metrics
        """
        logger.debug("Preparing to calculate metric %s from data:\n%s" % (self.name, data))
        if not self.virtual:
            return []
        else:
            results = []
            sets = {}
            for field in self.calculation.fields:
                logger.debug("Getting data for field `%s'" % field)
                for record in data.get((field, self.recipe_name, self.recipe_url), {'values':[]})['values']:
                    logger.debug("Data is `%s'" % record)
                    if record['content'] not in sets:
                        sets[record['context']] = {}
                    sets[record['context']][field] = record['value']
            logger.debug("Sets is %s" % sets)
            for context, field_data in sets.iteritems():
                if len(field_data) == len(self.calculation.fields):
                    logger.debug("Enough data to calculate %s; doing it..." % self.name)
                    args = [field_data[field] for field in self.calculation.fields]
                    value = self.calculation.run(*args)
                    results.append({'context':context, 'value':value})
                else:
                    logger.debug("Not enough data to calculate %s" % self.name)
            return results

    def values(self):
        """
        Used if this metric isn't virtual
        """
        logger.debug("Snapshotting metric %s" % self.name)
        return self._snapshot()

    def _container_for_context(self, context):
        if context in self.containers:
            return self.containers[context]
        else:
            container =  self._default_container_for(context)
            self.containers[context] = container
            return container

    def _default_container_for(self, context):
        return {"context" : context, "value" : 0}

    def _current_context(self, obj, *args, **kwargs):
        if self.context_finder:
            return self.context_finder(obj, *args, **kwargs)
        else:
            return None  

    def metadata(self):
        result = {}
        for field in ['name', 'unit', 'description', 'help_text', 'recipe_name', 'recipe_url']:
            result[field] = self.__dict__[field]
        result['data_type'] = self._data_type()
        return result

    def _instrument(self):
        pass
          
    def _record(self, func):
        """
        Call custom function and merge its result into containers
        """
        receiver = func
        args = []
        if isinstance(receiver, tuple):
            receiver = func[0]
            args = func[1:]
            result = receiver(*args)
            with self.lock:
                if isinstance(result, dict):
                    self.containers.update(result)
                else:
                    # TODO: Support context_finder & pre-defined contexts
                    container = self._container_for_context(None)
                    container["value"] = result
      
    def _snapshot(self):
        """
        Retrieve existing data for this metric, and reset it
        # TODO Make threadsafe
        """
        with self.lock:
            values = self.containers.values()
            self.containers = {}
            return values
    
class CounterMetric(Metric):

    def values(self):
        if self.virtual:
            return True
        if 'call' in self.options:
            self._record(self.options['call'])
        return self._snapshot()

    def _validate(self):
        if 'wrap' not in self.options and 'call' not in self.options:
            raise MetricError("Required `wrap' or `sources' option")

    def _instrument(self):
        self._wrap(self.options.get('wrap', []))

    def _wrap(self, target):
        if callable(target):
            logger.debug("%s wrapping %s" % (self, target))
            aspects.with_wrap(self._wrapper, target)
        else:
            for item in target:
                self._wrap(item)

    def _wrapper(self, obj, *args, **kwargs):
        with self.lock:
            context = self._current_context(obj, *args, **kwargs)
            container = self._container_for_context(context)
            container["value"] += 1
            yield aspects.proceed

    def _data_type(self):
        return 'counter'
  
        
class TimeMetric(Metric):

    def _default_container_for(self, context):
        return {"context" : context, "invocations" : 0, "value" : 0}

    def _validate(self):
        if self.virtual:
            return True
        if 'wrap' not in self.options:
            raise MetricError("Required `wrap'")

    def _instrument(self):
        self._wrap(self.options.get('wrap', []))

    def _wrap(self, target):
        if callable(target):
            logger.debug("%s wrapping %s" % (self, target))
            aspects.with_wrap(self._wrapper, target)
        else:
            for item in target:
                self._wrap(item)

    def _wrapper(self, obj, *args, **kwargs):
        with self.lock:
            context = self._current_context(obj, *args, **kwargs)
            container = self._container_for_context(context)
            start = time.time()
            yield aspects.proceed
            value = time.time() - start
            container["value"] += value
            container["invocations"] += 1

    def _data_type(self):
        return 'time'

class AbsoluteMetric(Metric):

    def _validate(self):
        if self.virtual:
            return True
        if 'call' not in self.options:
            raise MetricError("Required `call'")

    def values(self):
        self._record(self.options['call'])
        return self._snapshot()

    def _data_type(self):
        return 'absolute'

class PercentageMetric(Metric):

    def _validate(self):
        if self.virtual:
            return True
        if 'call' not in self.options:
            raise MetricError("Required `call'")

    def values(self):
        self._record(self.options['call'])
        return self._snapshot()

    def _data_type(self):
        return 'percentage'

class MetricSetting(object):

    def __init__(self):
        self.metrics = {}

    def time(self, name, *args, **options):
        "Add a time metric"
        self._add_metric(TimeMetric, name, *args, **options)

    def counter(self, name, *args, **options):
        "Add a counter metric"
        self._add_metric(CounterMetric, name, *args, **options)

    def absolute(self, name, *args, **options):
        "Add an absolute metric"
        self._add_metric(AbsoluteMetric, name, *args, **options)

    def percentage(self, name, *args, **options):
        "Add a percentage metric"
        self._add_metric(PercentageMetric, name, *args, **options)

    def add_recipe(self, name, url = None):
        """
        Add metrics from a recipe to this configuration.

        name -- A Recipe instance or String
        url -- A String, required to lookup the recipe if the name argument is not a Recipe instance
        """
        if isinstance(name, str):
            self.add_recipe(recipes.find(name, url))
        else:
            self._replay_recipe(name)

    def _replay_recipe(self, recipe):
        self.metrics.update(recipe.metrics)

    def _add_metric(self, metric_class, name, *args, **options):
        "Add a metric"
        metric =  metric_class(name, *args, **options)
        self.metrics[(metric.name, metric.recipe_name, metric.recipe_url)] = metric

import recipes



import logging
import os
import time
from fiveruns.dash.scm import handler

logger = logging.getLogger('fiveruns.dash.scm.handlers.svn')

class Handler(handler.Handler):

    def collect(self):
        self._check_info()
        if not len(self.data):
            return False
        self.data['scm_type'] = 'svn'
        return True

    def _check_info(self):
        try:
            fn = os.popen("svn info '%s'" % self.match)
        except:
            logger.warn("Could not access `svn info,` is `svn` in PATH?")
        mapping = {'URL': 'scm_url', 'Revision': 'scm_revision'}
        for line in fn.readlines():
            pair = [x.strip() for x in line.split(':', 1)]
            if mapping.has_key(pair[0]):
                self.data[mapping[pair[0]]] = pair[1]

def matches(directory):
    matcher = lambda dir, fn: os.path.isdir(os.path.join(dir, '.svn'))
    found = handler.ascend(matcher, directory, '')
    if found:
        return os.path.join(found, '.svn')



import logging
import os
import sys
import traceback

import aspects

import metrics
import recipes

logger = logging.getLogger('fiveruns.dash.metrics')

class Configuration(metrics.MetricSetting):

    def __init__(self, **options):
        super(Configuration, self).__init__()
        self.app_token = None
        # TODO: Extract these into a Session object
        self.vm_version = sys.version
        self.pid = os.getpid()
        self.pwd = os.getcwd()
        self.extra_payload_data = {}
        self.process_id = None
        self.report_interval = 60
        self.reporter = None
        self.__dict__.update(**options)

    def add_exceptions_from(self, target):
        logger.debug("Capturing exceptions from %s" % target)
        aspects.with_wrap(self._capture_exceptions, target) 

    def instrument(self):
        for metric in self.metrics.values():
            metric._instrument()

    def _capture_exceptions(self, obj):
        try:
            yield aspects.proceed
        except Exception, e:
            if self.reporter:
                try:
                    self.reporter.add_exception(e)                
                except:
                    logger.debug("Could not add exception due to internal error: %s\n%s" % (sys.exc_info()[1], "\n".join(traceback.format_tb(sys.exc_info()[2]))))
            raise

    def _replay_recipe(self, recipe):
        logger.debug("Adding %d metric(s) from recipe `%s' for %s to configuration" % (
            len(recipe.metrics),
            recipe.name, recipe.url))
        super(Configuration, self)._replay_recipe(recipe)



from __future__ import with_statement

import logging
import os
import time

from fiveruns.dash.scm import handler

logger = logging.getLogger('fiveruns.dash.scm.handlers.git')

class Handler(handler.Handler):

    def collect(self):
        git = self._import()
        if not git:
            return False

        repo = git.Repo(self.match)
        # check to see there are any branch heads at all
        if not repo.heads:
            head = None
        else:
            head = repo.commits('HEAD')

        # either there were no repo heads, or
        # the commit list is empty
        if not head:
            logger.warn("Could not retrieve Git HEAD info")
            return False

        tip = head[0]
        self.data.update({
            'scm_type': 'git',
            "scm_revision": tip.id,
            "scm_time": str(int(time.mktime(tip.authored_date)))
        })
        scm_url = self._find_origin(repo)
        if scm_url:
            self.data['scm_url'] = scm_url
        # TODO: Collect info and add to ``self.data``
        return True

    def _find_origin(self, repo):
        found_origin = False
        with open(os.path.join(repo.path, 'config'), 'r') as f:
            for line in f.readlines():
                if found_origin:
                    stripped = line.strip()
                    if stripped.startswith("url ="):
                        return stripped.split('=', 1)[1].strip()
                else:
                    if line.startswith('[remote "origin"]'):
                        found_origin = True
        

    def _import(self):
        """Attempt to import GitPython
        """
        try:
            import git_python
            return git_python
        except ImportError:
            try:
                import git
                return git
            except ImportError:
                self.missing('git_python')
                return False

def matches(directory):
    matcher = lambda dir, fn: os.path.isdir(os.path.join(dir, '.git'))
    found = handler.ascend(matcher, directory, '')
    if found:
        return os.path.join(found, '.git')



import logging
import sys
import traceback

logger = logging.getLogger('fiveruns.dash.exceptions')

class Recorder:

    def __init__(self, session):
        self.session = session
        self.data = {}

    def record(self, info, sample = {}):
        exc = self._extract(info)
        key = (exc['name'], exc['backtrace'])
        if key in self.data:
            self.data[key]['total'] += 1
        else:
            exc.update({"total": 1, "sample": self._flatten(sample)})
            self.data[key] = exc
        logger.debug("Recorded exception: %s" % self.data[key])

    def _extract(self, info):
        return {
            "name": type(info).__name__,
            "message": "\n".join(info.args),
            "backtrace": "Tracebacks are not currently supported for Python"
        }

    def _flatten(self, sample):
        return dict((str(k), str(v)) for k, v in sample.iteritems())



import os, logging

logger = logging.getLogger('fiveruns.dash.scm')

class Handler(object):

    def __init__(self, match):
        self.data = {}
        self.match = match
        self.collected = self.collect()

    def collect(self):
        """
        Collect the data for this SCM

        Needs to be defined in each handler or an exception is raised.
        """
        raise NotImplementedError("Abstract")

    def missing(self, pkg):
        logger.warn("Missing `%s' package.  Unable to report SCM details" % pkg)

    def __getitem__(self, key):
        return self.data[key]

    def keys(self):
        return self.data.keys()

def handlers():
    """
    Returns a list of all the SCM handlers that are available.
    """
    return [f[:-3] for f in os.listdir(os.path.join(os.path.dirname(__file__), 'handlers'))
            if not f.startswith('__') and f.endswith('.py')]

def at(directory):
    for name in handlers():
        match = _grab(name, 'matches')(directory)
        if match:
            logger.info("Found SCM: %s" % name[1:])
            handler = _grab(name, 'Handler')(match)
            if handler.collected:
                return handler
    logger.warn("Could not find SCM")

def _grab(name, attrib):
    return getattr(__import__('fiveruns.dash.scm.handlers.%s' % name, {}, {}, [attrib]), attrib)

def ascend(matcher, dirname, name):
    """
    Walk up a directory structure looking for a matching location
    """
    if matcher(dirname, name):
        return dirname
    else:
        if dirname in ('/', ''):
            return False
        else:
            return ascend(matcher, *os.path.split(dirname))




from __future__ import with_statement
from threading import Thread, RLock
import time, logging

import exceptions
from protocol import InfoPayload, DataPayload

logger = logging.getLogger('fiveruns.dash.session')

class Reporter(Thread):
    
    def __init__(self, config):
        self.config = config
        self.interval = config.report_interval
        self.tick = 1
        self.stop_requested = False
        self.reported_info = False
        self.exception_recorder = exceptions.Recorder(self)
        Thread.__init__(self)
        
    def stop(self, *args, **kwargs):
        logger.debug("Shutting down...")
        self.stop_requested = True
    
    def run(self):
        self._report_info()
        self.last_report = time.time()
        while not self.stop_requested:
            time.sleep(self.tick)
            if self.stop_requested: break 
            if self._is_ready():
                self._report_data()
        logger.debug("Shut down")

    def add_exception(self, info, sample={}):
        self.exception_recorder.record(info, sample)
                    
    def _is_ready(self):
        return time.time() - self.last_report > self.interval
        
    def _report_info(self):
        info = InfoPayload(self.config)
        logger.debug("Sending InfoPayload...")
        self.reported_info = info.send()
        return self.reported_info

    def _report_data(self):
        self.last_report = time.time()
        data = DataPayload(self.config)
        if self.reported_info or self._report_info():
            logger.debug("Sending DataPayload...")
            return data.send()
        else:
            logger.debug("Discarding data for this interval.")
            return False



import logging
import os
import sys

from configuration import Configuration
from recipes import Recipe
from session import Reporter

def start(config):
    config.instrument()
    reporter = Reporter(config)
    config.reporter = reporter
    reporter.start()
    return reporter
    
def configure(*args, **kwargs):
    """
    Returns a configuration with the arguments specified
    """
    return Configuration(*args, **kwargs)

def recipe(name, url):
    """
    Register and return a new recipe for name and url.

    >>> recipe1 = recipe('foo', 'http://example.com')
    >>> recipe1 # doctest: +ELLIPSIS
    <recipes.Recipe object at 0x...>
    >>> recipes.registry[('foo', 'http://example.com')] == recipe1
    True

    If a recipe is already defined by the name and url, a
    RegistrationError is raised

    >>> recipes.registry[('foo', 'http://example.com')] == recipe1
    True
    >>> recipe('foo', 'http://example.com')
    Traceback (most recent call last):
        ...
    DuplicateRecipe: `foo' defined for `http://example.com'
    """
    return Recipe(name, url)
    
class NullLoggingHandler(logging.Handler):
    def emit(self, record):
        pass

def _configure_logger():
    logger = logging.getLogger("fiveruns.dash")
    stream = logging.StreamHandler()
    logger.addHandler(stream)
    formatter = logging.Formatter("%(name)s [%(levelname)s] %(message)s")
    stream.setFormatter(formatter)
    logger.setLevel(logging.WARN)
    return logger

logger = _configure_logger()



from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages
import fiveruns.dash.version

setup(
  name = "fiveruns.dash",
  version = ".".join(str(s) for s in fiveruns.dash.version.info),
  description = "FiveRuns Dash base library for Python",
  author = "FiveRuns Development Team",
  author_email = "dev@fiveruns.com",
  url = "http://dash.fiveruns.com",
  packages = find_packages(exclude = "tests"),
  install_requires = ['simplejson', 'python-aspects >= 1.3'],
  license = 'MIT',
  dependency_links = [
    # Pypi only has python-aspects-1.1 as of 2009-01-19, need 1.3
    'http://www.cs.tut.fi/~ask/aspects/python-aspects-1.3.tar.gz'
  ],
  namespace_packages = ['fiveruns'],
  extras_require = {
    'examples':  ["Twisted"]
  },
  keywords = "monitoring performance production metrics logging"
)
